﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Threading.Tasks;

using TestProject.Models;

namespace TestProject.Pages
{
    /// <summary>
    /// 인덱스 모델
    /// </summary>
    public class IndexModel : PageModel
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 할일 클라이언트
        /// </summary>
        private readonly TodoClient client;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 완료 할일 리스트 - CompleteTodoList

        /// <summary>
        /// 완료 할일 리스트
        /// </summary>
        public List<TodoModel> CompleteTodoList { get; private set; }

        #endregion
        #region 미완료 할일 리스트 - IncompleteTodoList

        /// <summary>
        /// 미완료 할일 리스트
        /// </summary>
        public List<TodoModel> IncompleteTodoList { get; private set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - IndexModel(client)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="client">할일 클라이언트</param>
        public IndexModel(TodoClient client)
        {
            this.client = client;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region GET 요청시 처리하기 (비동기) - OnGetAsync()

        /// <summary>
        /// GET 요청시 처리하기 (비동기)
        /// </summary>
        /// <returns>태스크</returns>
        public async Task OnGetAsync()
        {
            List<TodoModel> todoList = await this.client.GetListAsync();

            CompleteTodoList   = todoList.Where(x => x.IsComplete).ToList();
            IncompleteTodoList = todoList.Except(CompleteTodoList).ToList();
        }

        #endregion
        #region POST 생성 요청시 처리하기 (비동기) - OnPostCreateAsync(name)

        /// <summary>
        /// POST 생성 요청시 처리하기 (비동기)
        /// </summary>
        /// <param name="name">명칭</param>
        /// <returns>액션 결과 태스크</returns>
        public async Task<IActionResult> OnPostCreateAsync([Required] string name)
        {
            if(!ModelState.IsValid)
            {
                return RedirectToPage();
            }

            TodoModel newTodo = new TodoModel
            {
                Name = name
            };

            await this.client.CreateItemAsync(newTodo);

            return RedirectToPage();
        }

        #endregion
    }
}